import os, mne, sys
import os.path as op
import numpy as np
import numpy.fft as fft
import pandas as pd
import matplotlib.pyplot as plt
from matplotlib.ticker import FormatStrFormatter
import glob
from mne.viz import plot_compare_evokeds as pce


# Stefan
#dir_main = op.join(op.expanduser("~"),'Documents','Box Sync', 'ASSR2')
# Malina
dir_main = op.join(op.expanduser("~"),'Box Sync', 'ASSR2')
# EEG lab
#dir_main = op.join(op.expanduser("~"), 'Documents', 'Malina_Box','Box Sync', 'ASSR2') # EEG lab

sys.path.append(dir_main + '\\MNE')

#mne.set_log_level('WARNING')

def ERP(c, evodict):
    '''
    take the individual ERP data for a single condition and return a list
    note that list(evodict[c]) gives all subject numbers in a list
    Arguments:
        c -- key from evodict dictionary (experimental condition)
        evodict -- dictionary with experimental conditions, each key in evodict 
            is a dictionary itself, with participant's number (key) 
            and evoked arrays (value)
    '''
    return [evodict[c][i] for i in list(evodict[c])]

def extract_mean_amps(d, t, ch, fp, cond):
    '''
    extract mean amplitudes across time and channels
    saved in V, convert to microV
    Arguments:
        d -- dictionary with experimental conditions, each key in evodict 
            is a dictionary itself, with participant's number (key) 
            and evoked arrays (value)
        t -- samples from which to extract the mean amps 
        ch -- channels from which to extract the mean amps
        fp -- participant number
        cond -- experimental condition (must match one of the keys in d)
    '''
    return np.mean([dx[t] for dx in d[cond][fp].data[ch]]) * 1e6


# Set file paths
dir_results = op.join(dir_main, 'results')
dir_evo = op.join(dir_main, 'MNE', 'fif', 'evokeds')
dir_epo = op.join(dir_main, 'MNE', 'fif', 'epochs')

# ========================
# Process auditory data
# ========================
# data are saved as epochs
# Load filenames
os.chdir(dir_epo)  # change directory to averages folder
files = glob.glob('*audio-epo.fif')

# same condition information as in preprocess file
conds = {'no20_1': 121, 'no20_2': 122,
          'no40_1': 141, 'no40_2': 142,
          'no80_1': 181, 'no80_2': 182,
          'lo20_1': 221, 'lo20_2': 222,
          'lo40_1': 241, 'lo40_2': 242,
          'lo80_1': 281, 'lo80_2': 282,
          'hi20_1': 321, 'hi20_2': 322,
          'hi40_1': 341, 'hi40_2': 342,
          'hi80_1': 381, 'hi80_2': 382}
conds = list(conds)

# We have these electrodes:
# ['Nz', 'Fpz', 'Fz', 'FCz', 'Cz', 'CPz', 'Pz', 'Cheek', 'EOG']
# Use Fz and FCz for audio
chanpick = ['Fz', 'FCz']

# save evoked (i.e., averages)
evo_audio = {i:{} for i in conds}

fps = []
# save data by subject (for export)
Dall = [] # save all channels
Dsel = [] # save only signal and noise channels
# save data by condition (for figures)
Damp_all = {i:[] for i in conds} # amplitudes for the wide frequency spectrum
Damp = {i:[] for i in conds} # amplitudes for the selected frequency spectrum
Damp_indEpo = {i:[] for i in conds} # amplitudes when first FFT and then avg
Ditc = {i:[] for i in conds} # intertrial coherence for selected freq spectrum

## if you only want to run the participants after preregistration:
# files = files[8:len(files)]
for f in files:
    # f = files[0]
    print('\n'+'='*50)
    fp = int(f[2:4])
    print('Subject:', fp)
    print('='*50)
    fps += [fp]
    epo_fp = mne.read_epochs(f, preload=True)
    tmpall = {'fp': fp} # save all channels
    tmpsel = {'fp': fp} # save only signal and noise channels
    for cond in conds:
        #cond = 'no40_1'
        # compute GM (ie evokeds)
        # =========================
        ev = epo_fp[cond].average()
        #ev.set_eeg_reference(ref_channels=['Nz'])
        ev.info['subject_info'] = fp
        print(cond + " = " + str(ev.nave) + " trials")
        evo_audio[cond][fp] = ev
        chans = [ev.ch_names.index(c) for c in chanpick]

        # use fft from numpy
        # ==================
        # select frequencies (by defining min and max below)
        Fs = ev.info['sfreq']
        fftfreq = fft.fftfreq(len(ev.times), 1/Fs)

        if np.any((cond == 'no80_1', cond == 'no80_2',
                  cond == 'lo80_1', cond == 'lo80_2',
                  cond == 'hi80_1', cond == 'hi80_2'), axis = 0):
            ASSRfreq = 81.92
            myfreqs_80 = fftfreq[np.logical_and(fftfreq > ASSRfreq-15, fftfreq < ASSRfreq+15)]

        if np.any((cond == 'no20_1', cond == 'no20_2',
                  cond == 'lo20_1', cond == 'lo20_2',
                  cond == 'hi20_1', cond == 'hi20_2'), axis = 0):
            ASSRfreq = 20.48
            myfreqs_20 = fftfreq[np.logical_and(fftfreq > ASSRfreq-15, fftfreq < ASSRfreq+15)]

        if np.any((cond == 'no40_1', cond == 'no40_2',
                  cond == 'lo40_1', cond == 'lo40_2',
                  cond == 'hi40_1', cond == 'hi40_2'), axis = 0):
            ASSRfreq = 40.96
            myfreqs_40 = fftfreq[np.logical_and(fftfreq > ASSRfreq-15, fftfreq < ASSRfreq+15)]

        myfreqs_all = fftfreq[9:160] # wide freq spectrum, up to  bit more than 100 Hz
        freqsindx_all = np.where(np.isin(fftfreq,myfreqs_all))[0]

        myfreqs = fftfreq[np.logical_and(fftfreq > ASSRfreq-15, fftfreq < ASSRfreq+15)]
        freqsindx = np.where(np.isin(fftfreq,myfreqs))[0]

        # index signal and noise frequencies
        indx_signal = int(np.where(myfreqs == ASSRfreq)[0])
        # how many neighboring noise frequencies?
        nSNRfreqs = 8
        sur_lo = list(range(indx_signal-2-nSNRfreqs, indx_signal-2, 1))
        sur_hi = list(range(indx_signal+3, indx_signal+3+nSNRfreqs, 1))
        indx_noise = sur_lo + sur_hi
        # myfreqs[indx_noise]
        # for 20 Hz: [14.08, 14.72, 15.36, 16.  , 16.64, 17.28, 17.92, 18.56, 22.4, 23.04, 23.68, 24.32, 24.96, 25.6 , 26.24, 26.88]
        # for 40 Hz: [34.56, 35.2 , 35.84, 36.48, 37.12, 37.76, 38.4 , 39.04, 42.88, 43.52, 44.16, 44.8 , 45.44, 46.08, 46.72, 47.36]
        # for 80 Hz: [75.52, 76.16, 76.8 , 77.44, 78.08, 78.72, 79.36, 80., 83.84, 84.48, 85.12, 85.76, 86.4 , 87.04, 87.68, 88.32]
        
        # extract amplitude spectrum for GM (ie evokeds)
        fftamps = np.abs(fft.fft(ev.data[chans], axis = 1))*1e3
        fftamps_all = fftamps[:,freqsindx_all]
        fftamps = fftamps[:,freqsindx]
        
        fft_indEpo = np.mean(np.abs(fft.fft(epo_fp[cond].get_data()[:,chans,:], axis = 1))*1e3, axis = 0)
        fft_indEpo = fft_indEpo[:,freqsindx_all]

        Damp[cond].append(np.mean(fftamps, axis = 0))
        Damp_all[cond].append(np.mean(fftamps_all, axis = 0))
        Damp_indEpo[cond].append(np.mean(fft_indEpo, axis = 0))

        for c in range(0,len(chanpick)):
            for i in range(0,len(myfreqs)):
                tmpall['Amp%s_%s_%.2f'%(cond, chanpick[c],myfreqs[i])] = fftamps[c,i]

        # S, N, SNR, and SmN for all channels
        tmpS = fftamps[:,indx_signal]
        tmpN = np.mean(fftamps[:,indx_noise],axis=1)
        tmpSNR = tmpS/tmpN
        tmpS = np.mean(tmpS)
        tmpN = np.mean(tmpN)
        tmpSNR = np.mean(tmpSNR)
        tmpsel['AmpS'+cond] = tmpS
        tmpsel['AmpN'+cond] = tmpN
        tmpsel['AmpSNR'+cond] = tmpSNR
        tmpsel['AmpSmN'+cond] = tmpS - tmpN

        # extract itc from individual trials (ie epocheds)
        # =================================================
        # fft results for selected channels
        F = fft.fft(epo_fp[cond].get_data()[:,chans,:], axis=2)
        # the number of epochs
        N = len(F)
        # some computations ...
        fftitc = F/np.abs(F)
        fftitc = np.sum(fftitc, 0)
        fftitc = np.abs(fftitc)/N

        # extract selected channels and freq of interest
        fftitc = fftitc[:,freqsindx]
        Ditc[cond].append(np.mean(fftitc, axis = 0))

        for c in range(0,len(chanpick)):
            for i in range(0,len(myfreqs)):
                tmpall['Itc%s_%.2f'%(chanpick[c],myfreqs[i])] = fftitc[c,i]

        # S, N, SNR, and SmN for all channels
        tmpS = fftitc[:,indx_signal]
        tmpN = np.mean(fftitc[:,indx_noise],axis=1)
        tmpSNR = tmpS/tmpN
        tmpS = np.mean(tmpS)
        tmpN = np.mean(tmpN)
        tmpSNR = np.mean(tmpSNR)
        tmpsel['ItcS'+cond] = tmpS
        tmpsel['ItcN'+cond] = tmpN
        tmpsel['ItcSNR'+cond] = tmpSNR
        tmpsel['ItcSmN'+cond] = tmpS - tmpN
    Dall += [tmpall]
    Dsel += [tmpsel]
for i in Damp:
    Damp[i] = np.row_stack(Damp[i])
for i in Damp_all:
    Damp_all[i] = np.row_stack(Damp_all[i])
for i in Damp_indEpo:
    Damp_indEpo[i] = np.row_stack(Damp_indEpo[i])
for i in Ditc:
    Ditc[i] = np.row_stack(Ditc[i])

df = pd.DataFrame(Dall)
df.to_csv(op.join(dir_results, 'data_audio_all_channels_N_{}.tsv'.format(len(fps))), sep='\t', index=False)
df = pd.DataFrame(Dsel)
df.to_csv(op.join(dir_results, 'data_audio_signal_noise_N_{}.tsv'.format(len(fps))), sep='\t', index=False)
print('Done with ', len(fps), ' subjects:\n',fps)

# compute grand means across blocks
# =================================
evo_audioGM = {'no20': {}, 'no40': {},'no80': {},
               'lo20': {}, 'lo40': {},'lo80': {},
               'hi20': {}, 'hi40': {},'hi80': {}}


for fp in fps:
    evo_audioGM['no20'][fp] = mne.combine_evoked([evo_audio[i][fp] for i in conds[0:2]],weights = [0.5, 0.5])
    evo_audioGM['no40'][fp] = mne.combine_evoked([evo_audio[i][fp] for i in conds[2:4]],weights = [0.5, 0.5])
    evo_audioGM['no80'][fp] = mne.combine_evoked([evo_audio[i][fp] for i in conds[4:6]],weights = [0.5, 0.5])
    evo_audioGM['lo20'][fp] = mne.combine_evoked([evo_audio[i][fp] for i in conds[6:8]],weights = [0.5, 0.5])
    evo_audioGM['lo40'][fp] = mne.combine_evoked([evo_audio[i][fp] for i in conds[8:10]],weights = [0.5, 0.5])
    evo_audioGM['lo80'][fp] = mne.combine_evoked([evo_audio[i][fp] for i in conds[10:12]],weights = [0.5, 0.5])
    evo_audioGM['hi20'][fp] = mne.combine_evoked([evo_audio[i][fp] for i in conds[12:14]],weights = [0.5, 0.5])
    evo_audioGM['hi40'][fp] = mne.combine_evoked([evo_audio[i][fp] for i in conds[14:16]],weights = [0.5, 0.5])
    evo_audioGM['hi80'][fp] = mne.combine_evoked([evo_audio[i][fp] for i in conds[16:18]],weights = [0.5, 0.5])
    # when I chose weights = 'equal', the weights were +1 each and the result was a sum of the ERPs


# plot mean amps and amplitude spectra for individual subjects
# ============================================================
#ylim = [-3, 3] # force a particular Y min and max
style = dict(colors={'No': 'black', 'Low': 'blue', 'High': 'red'},
    ci=None, show_sensors=False, legend='upper right',
    linestyles={'No': '-', 'Low': '-', 'High': '-'},
    truncate_xaxis=False, truncate_yaxis=False) #, ylim=dict(eeg=ylim))
plt.rcParams.update({'font.size': 14})

for fp in fps:
    # fp = 5
    tmp_myfreqs = [myfreqs_20, myfreqs_40, myfreqs_80]
    tmp_names = [20,40,80]
                    # 20 no lo hi,  40 no lo hi,  80 no lo hi
    tmp_whichCon = [[0,2,6,8,12,14],[2,4,8,10,14,16],[4,6,10,12,16,18]]
    for f in range(3):
        # f = 0
        fig, ax = plt.subplots(1, 2, figsize=(14, 5), sharey=False)
        pce({'No':  evo_audioGM['no{}'.format(tmp_names[f])][fp],
             'Low': evo_audioGM['lo{}'.format(tmp_names[f])][fp],
             'High': evo_audioGM['hi{}'.format(tmp_names[f])][fp]},
             **style, picks=chans, combine = 'mean',
             axes = ax[0])
        fpind = fps.index(fp)
        # for each load, combine the two blocks
        # same condition information as in preprocess file
#        conds = {'no20_1': 121, 'no20_2': 122,
#                  'no40_1': 141, 'no40_2': 142,
#                  'no80_1': 181, 'no80_2': 182,
#                  'lo20_1': 221, 'lo20_2': 222,
#                  'lo40_1': 241, 'lo40_2': 242,
#                  'lo80_1': 281, 'lo80_2': 282,
#                  'hi20_1': 321, 'hi20_2': 322,
#                  'hi40_1': 341, 'hi40_2': 342,
#                  'hi80_1': 381, 'hi80_2': 382}

        tmpno = np.mean(np.row_stack([Damp[c][fps.index(fp)] for c in conds[tmp_whichCon[f][0]:tmp_whichCon[f][1]]]), axis=0)
        tmplo = np.mean(np.row_stack([Damp[c][fps.index(fp)] for c in conds[tmp_whichCon[f][2]:tmp_whichCon[f][3]]]), axis=0)
        tmphi = np.mean(np.row_stack([Damp[c][fps.index(fp)] for c in conds[tmp_whichCon[f][4]:tmp_whichCon[f][5]]]), axis=0)

        ax[1].plot(tmp_myfreqs[f], tmpno, 'k-', label='No')
        ax[1].plot(tmp_myfreqs[f], tmplo, 'b-', label='Low')
        ax[1].plot(tmp_myfreqs[f], tmphi, 'r-', label='High')
        ax[1].legend(frameon=False)
        ax[1].set_xlabel('Frequencies (Hz)')
        ax[1].set_ylabel('Amplitudes (µV)')
        ax[1].spines['top'].set_visible(False)
        ax[1].spines['right'].set_visible(False)
        ax[1].yaxis.set_major_formatter(FormatStrFormatter('%.2f'))
        ax[1].set_title("Amplitude spectrum")
        #tmp_myfreqs[f][indx_noise[7]]
        ax[1].axvspan(min(tmp_myfreqs[f][indx_noise]), tmp_myfreqs[f][indx_noise[7]], color='gray', alpha=0.2, lw=0)
        ax[1].axvspan(tmp_myfreqs[f][indx_noise[8]], max(tmp_myfreqs[f][indx_noise]), color='gray', alpha=0.2, lw=0)
        fig.suptitle("Subject "+str(fp))
        fig.tight_layout()
        plt.savefig(op.join(dir_results, 'figures', 'individual_aspctm', 'fp_{}_for_{}Hz_AM.jpg'.format(fp, tmp_names[f])), format='jpg', dpi = 600)
        plt.savefig(op.join(dir_results, 'figures', 'individual_aspctm', 'fp_{}_for_{}Hz_AM.pdf'.format(fp, tmp_names[f])), format='pdf', dpi = 600)
        plt.close(fig=None)

        fig, ax = plt.subplots(1, 2, figsize=(14, 5), sharey=False)
        pce({'No':  evo_audioGM['no{}'.format(tmp_names[f])][fp],
             'Low': evo_audioGM['lo{}'.format(tmp_names[f])][fp],
             'High': evo_audioGM['hi{}'.format(tmp_names[f])][fp]},
             **style, picks=chans, combine = 'mean',
             axes = ax[0])
        fpind = fps.index(fp)

        # For debuggin reasons, plot spectrum in frequency range from 5 to 100 Hz:
        tmpno = np.mean(np.row_stack([Damp_all[c][fps.index(fp)] for c in conds[tmp_whichCon[f][0]:tmp_whichCon[f][1]]]), axis=0)
        tmplo = np.mean(np.row_stack([Damp_all[c][fps.index(fp)] for c in conds[tmp_whichCon[f][2]:tmp_whichCon[f][3]]]), axis=0)
        tmphi = np.mean(np.row_stack([Damp_all[c][fps.index(fp)] for c in conds[tmp_whichCon[f][4]:tmp_whichCon[f][5]]]), axis=0)

        ax[1].plot(myfreqs_all, tmpno, 'k-', label='No')
        ax[1].plot(myfreqs_all, tmplo, 'b-', label='Low')
        ax[1].plot(myfreqs_all, tmphi, 'r-', label='High')
        ax[1].legend(frameon=False)
        ax[1].set_xlabel('Frequencies (Hz)')
        ax[1].set_ylabel('Amplitudes (µV)')
        ax[1].spines['top'].set_visible(False)
        ax[1].spines['right'].set_visible(False)
        ax[1].yaxis.set_major_formatter(FormatStrFormatter('%.2f'))
        ax[1].set_title("Amplitude spectrum")
        ax[1].axvspan(min(tmp_myfreqs[f][indx_noise]), tmp_myfreqs[f][indx_noise[7]], color='gray', alpha=0.2, lw=0)
        ax[1].axvspan(tmp_myfreqs[f][indx_noise[8]], max(tmp_myfreqs[f][indx_noise]), color='gray', alpha=0.2, lw=0)
        fig.suptitle("Subject "+str(fp))
        fig.tight_layout()
        plt.savefig(op.join(dir_results, 'figures', 'individual_aspctm', 'fp_{}_for_{}Hz_AM_wide.jpg'.format(fp, tmp_names[f])), format='jpg', dpi = 600)
        plt.savefig(op.join(dir_results, 'figures', 'individual_aspctm', 'fp_{}_for_{}Hz_AM_wide.pdf'.format(fp, tmp_names[f])), format='pdf', dpi = 600)
        plt.close(fig=None)

# plot grand mean amplitude spectrum by load
# ==========================================
for f in range(3):
    #f = 0
    tmp_myfreqs = [myfreqs_20, myfreqs_40, myfreqs_80]
    tmp_names = [20,40,80]
    tmp_whichCon = [[0,2,6,8,12,14],[2,4,8,10,14,16],[4,6,10,12,16,18]]

    plt.close(fig=None)
    fig, ax = plt.subplots(2, 1, figsize=(14, 9), sharey=False)
    style = dict(show_sensors=False, legend='upper right',
                 truncate_xaxis=False, truncate_yaxis=False, ci=None)
    pce({'No': ERP('no{}'.format(tmp_names[f]), evo_audioGM),
         'Low': ERP('lo{}'.format(tmp_names[f]), evo_audioGM),
         'High': ERP('hi{}'.format(tmp_names[f]), evo_audioGM)},
         colors={'No': 'black', 'Low': 'blue', 'High': 'red'},
         linestyles={'No': '-', 'Low': '-', 'High': '-'},
         **style, title=", ".join(chanpick), picks=chans, combine = 'mean',
         axes = ax[0])
        #ylim=dict(eeg=[-6, 12])
    ax[0].set_title('Mean ERP (top) and amplitude spectrum (bottom) (N = {})'.format(len(fps)))
    ax[0].set_ylabel('Amplitudes (µV)')
    ax[0].legend(frameon=False)
    ax[0].yaxis.set_major_formatter(FormatStrFormatter('%.2f'))
    
    # for each load, combine the two blocks
    tmpno = np.mean(np.row_stack([np.row_stack([Damp[c][fps.index(fp)] for c in conds[tmp_whichCon[f][0]:tmp_whichCon[f][1]]]) for fp in fps]), axis=0)
    tmplo = np.mean(np.row_stack([np.row_stack([Damp[c][fps.index(fp)] for c in conds[tmp_whichCon[f][2]:tmp_whichCon[f][3]]]) for fp in fps]), axis=0)
    tmphi = np.mean(np.row_stack([np.row_stack([Damp[c][fps.index(fp)] for c in conds[tmp_whichCon[f][4]:tmp_whichCon[f][5]]]) for fp in fps]), axis=0)

    ax[1].plot(tmp_myfreqs[f], tmpno, 'k-', label='No')
    ax[1].plot(tmp_myfreqs[f], tmplo, 'b-', label='Low')
    ax[1].plot(tmp_myfreqs[f], tmphi, 'r-', label='High')
    ax[1].legend(frameon=False)
    ax[1].set_xlabel('Frequencies (Hz)')
    ax[1].set_ylabel('Amplitudes (µV)')
    ax[1].spines['top'].set_visible(False)
    ax[1].spines['right'].set_visible(False)
    ax[1].yaxis.set_major_formatter(FormatStrFormatter('%.2f'))
    ax[1].axvspan(min(tmp_myfreqs[f][indx_noise]), tmp_myfreqs[f][indx_noise[7]], color='gray', alpha=0.2, lw=0)
    ax[1].axvspan(tmp_myfreqs[f][indx_noise[8]], max(tmp_myfreqs[f][indx_noise]), color='gray', alpha=0.2, lw=0)
    fig.tight_layout()
    fig.savefig(op.join(dir_results, 'figures', 'figure_average_aspctm_and_ERP_for_{}Hz_N_{}.jpg'.format(tmp_names[f], len(fps))), format='jpg', dpi=600)
    fig.savefig(op.join(dir_results, 'figures', 'figure_average_aspctm_and_ERP_for_{}Hz_N_{}.pdf'.format(tmp_names[f], len(fps))), format='pdf', dpi=600)
    plt.close(fig=None)

    fig, ax = plt.subplots(2, 1, figsize=(14, 9), sharey=False)
    style = dict(show_sensors=False, legend='upper right',
                 truncate_xaxis=False, truncate_yaxis=False, ci=None)
    pce({'No': ERP('no{}'.format(tmp_names[f]), evo_audioGM),
         'Low': ERP('lo{}'.format(tmp_names[f]), evo_audioGM),
         'High': ERP('hi{}'.format(tmp_names[f]), evo_audioGM)},
         colors={'No': 'black', 'Low': 'blue', 'High': 'red'},
         linestyles={'No': '-', 'Low': '-', 'High': '-'},
         **style, title=", ".join(chanpick), picks=chans, combine = 'mean',
         axes = ax[0])
        #ylim=dict(eeg=[-6, 12])
    ax[0].set_title('Mean ERP (top) and amplitude spectrum (bottom) (N = %i)' % len(fps))
    ax[0].set_ylabel('Amplitudes (µV)')
    ax[0].legend(frameon=False)
    ax[0].yaxis.set_major_formatter(FormatStrFormatter('%.2f'))

    #For debuggin reasons, plot spectrum in frequency range from 5 to 100 Hz:
    tmpno = np.mean(np.row_stack([np.row_stack([Damp_all[c][fps.index(fp)] for c in conds[tmp_whichCon[f][0]:tmp_whichCon[f][1]]]) for fp in fps]), axis=0)
    tmplo = np.mean(np.row_stack([np.row_stack([Damp_all[c][fps.index(fp)] for c in conds[tmp_whichCon[f][2]:tmp_whichCon[f][3]]]) for fp in fps]), axis=0)
    tmphi = np.mean(np.row_stack([np.row_stack([Damp_all[c][fps.index(fp)] for c in conds[tmp_whichCon[f][4]:tmp_whichCon[f][5]]]) for fp in fps]), axis=0)

    ax[1].plot(myfreqs_all, tmpno, 'k-', label='No')
    ax[1].plot(myfreqs_all, tmplo, 'b-', label='Low')
    ax[1].plot(myfreqs_all, tmphi, 'r-', label='High')
    ax[1].legend(frameon=False)
    ax[1].set_xlabel('Frequencies (Hz)')
    ax[1].set_ylabel('Amplitudes (µV)')
    ax[1].spines['top'].set_visible(False)
    ax[1].spines['right'].set_visible(False)
    ax[1].yaxis.set_major_formatter(FormatStrFormatter('%.2f'))
    ax[1].axvspan(min(tmp_myfreqs[f][indx_noise]), tmp_myfreqs[f][indx_noise[7]], color='gray', alpha=0.2, lw=0)
    ax[1].axvspan(tmp_myfreqs[f][indx_noise[8]], max(tmp_myfreqs[f][indx_noise]), color='gray', alpha=0.2, lw=0)
    fig.tight_layout()
    fig.savefig(op.join(dir_results, 'figures', 'figure_average_aspctm_and_ERP_for_{}Hz_wideband_N_{}.jpg'.format(tmp_names[f],len(fps))), format='jpg', dpi=600)
    fig.savefig(op.join(dir_results, 'figures', 'figure_average_aspctm_and_ERP_for_{}Hz_wideband_N_{}.pdf'.format(tmp_names[f],len(fps))), format='pdf', dpi=600)
    plt.close(fig=None)


# plot grand mean itc by load
# ===========================
for f in range(3):
    #f = 0
    tmp_myfreqs = [myfreqs_20, myfreqs_40, myfreqs_80]
    tmp_names = [20,40,80]
    tmp_whichCon = [[0,2,6,8,12,14],[2,4,8,10,14,16],[4,6,10,12,16,18]]

    plt.close(fig=None)
    fig, ax = plt.subplots(figsize=(14, 9), sharey=False) #subplot seems to be needed to define spines below
    # for each load, combine the four blocks
    tmpno = np.mean(np.row_stack([np.row_stack([Ditc[c][fps.index(fp)] for c in conds[tmp_whichCon[f][0]:tmp_whichCon[f][1]]]) for fp in fps]), axis=0)
    tmplo = np.mean(np.row_stack([np.row_stack([Ditc[c][fps.index(fp)] for c in conds[tmp_whichCon[f][2]:tmp_whichCon[f][3]]]) for fp in fps]), axis=0)
    tmphi = np.mean(np.row_stack([np.row_stack([Ditc[c][fps.index(fp)] for c in conds[tmp_whichCon[f][4]:tmp_whichCon[f][5]]]) for fp in fps]), axis=0)
    plt.plot(tmp_myfreqs[f], tmpno, 'k-', label='No')
    plt.plot(tmp_myfreqs[f], tmplo, 'b-', label='Low')
    plt.plot(tmp_myfreqs[f], tmphi, 'r-', label='High')
    plt.title('Intertrial coherence (itc)')
    plt.legend(frameon=False)
    plt.xlabel('Frequencies (Hz)')
    plt.ylabel('itc')
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.axvspan(min(tmp_myfreqs[f][indx_noise]), tmp_myfreqs[f][indx_noise[7]], color='gray', alpha=0.2, lw=0)
    ax.axvspan(tmp_myfreqs[f][indx_noise[8]], max(tmp_myfreqs[f][indx_noise]), color='gray', alpha=0.2, lw=0)
    fig.tight_layout()
    fig.savefig(op.join(dir_results, 'figures', 'figure_audio_itc_for_{}_N_{}.jpg'.format(tmp_names[f],len(fps))), format='jpg', dpi=600)
    fig.savefig(op.join(dir_results, 'figures', 'figure_audio_itc_for_{}_N_{}.pdf'.format(tmp_names[f],len(fps))), format='pdf', dpi=600)
    plt.close(fig=None)


# ========================
# Process visual data
# ========================
    
# Load filenames
os.chdir(dir_evo)  # change directory to averages folder
files = glob.glob('*_visual_no_non20_1-ave.fif')

## if you only want to run the participants after preregistration:
# files = files[8:len(files)]

conds = {'no_non20_1': 121, 'no_non20_2': 122,
                   'no_non40_1': 141, 'no_non40_2': 142,
                   'no_non80_1': 181, 'no_non80_2': 182,
                   'lo_non20_1': 221, 'lo_non20_2': 222,
                   'lo_non40_1': 241, 'lo_non40_2': 242,
                   'lo_non80_1': 281, 'lo_non80_2': 282,
                   'lo_tar20_1': 321, 'lo_tar20_2': 322,
                   'lo_tar40_1': 341, 'lo_tar40_2': 342,
                   'lo_tar80_1': 381, 'lo_tar80_2': 382,
                   'hi_non20_1': 421, 'hi_non20_2': 422,
                   'hi_non80_1': 441, 'hi_non40_2': 442,
                   'hi_non40_1': 481, 'hi_non80_2': 482,
                   'hi_tar20_1': 521, 'hi_tar20_2': 522,
                   'hi_tar40_1': 541, 'hi_tar40_2': 542,
                   'hi_tar80_1': 581, 'hi_tar80_2': 582}
conds = list(conds)

# compute ERPs per condition per subject
# ======================================
evo_visual = {i:{} for i in conds}
fps = []
for f in files:
    # f = files[0]
    print('\n'+'='*50)
    fp = int(f[2:4])
    print('Subject:', fp)
    print('='*50)
    fps += [fp]
    for c in conds:
        ev = mne.read_evokeds(op.join(dir_main, 'MNE', 'fif', 'evokeds',
                                      'fp%02d_visual_' % fp + c + '-ave.fif' ), condition = c)
        print(ev.comment+" = "+str(ev.nave))
        ev.info['subject_info'] = fp
        ev.set_eeg_reference(ref_channels=['Nz'])
        ev.filter(h_freq=30, l_freq=None)
        ev.apply_baseline(baseline=(None, 0))
        # from beginning of interval to onset; so, from -100 to 0
        evo_visual[ev.comment][fp] = ev

# compute grand means across blocks by load and target/nontarget
# ==============================================================
evo_visualGM = {'lo_tar20': {}, 'lo_tar40': {}, 'lo_tar80': {},
                'hi_tar20': {}, 'hi_tar40': {}, 'hi_tar80': {},
                'no_non20': {}, 'no_non40': {}, 'no_non80': {},
                'lo_non20': {}, 'lo_non40': {}, 'lo_non80': {},
                'hi_non20': {}, 'hi_non40': {}, 'hi_non80': {}}

for fp in fps:
    evo_visualGM['lo_tar20'][fp] = mne.combine_evoked([evo_visual[conds[i]][fp]
        for i in np.where(['lo_tar20' in c for c in conds])[0]], weights = [0.5, 0.5])
    evo_visualGM['lo_tar40'][fp] = mne.combine_evoked([evo_visual[conds[i]][fp]
        for i in np.where(['lo_tar40' in c for c in conds])[0]], weights = [0.5, 0.5])
    evo_visualGM['lo_tar80'][fp] = mne.combine_evoked([evo_visual[conds[i]][fp]
        for i in np.where(['lo_tar80' in c for c in conds])[0]], weights = [0.5, 0.5])
    evo_visualGM['hi_tar20'][fp] = mne.combine_evoked([evo_visual[conds[i]][fp]
        for i in np.where(['hi_tar20' in c for c in conds])[0]], weights = [0.5, 0.5])
    evo_visualGM['hi_tar40'][fp] = mne.combine_evoked([evo_visual[conds[i]][fp]
        for i in np.where(['hi_tar40' in c for c in conds])[0]], weights = [0.5, 0.5])
    evo_visualGM['hi_tar80'][fp] = mne.combine_evoked([evo_visual[conds[i]][fp]
        for i in np.where(['hi_tar80' in c for c in conds])[0]], weights = [0.5, 0.5])
    evo_visualGM['no_non20'][fp] = mne.combine_evoked([evo_visual[conds[i]][fp]
        for i in np.where(['no_non20' in c for c in conds])[0]], weights = [0.5, 0.5])
    evo_visualGM['no_non40'][fp] = mne.combine_evoked([evo_visual[conds[i]][fp]
        for i in np.where(['no_non40' in c for c in conds])[0]], weights = [0.5, 0.5])
    evo_visualGM['no_non80'][fp] = mne.combine_evoked([evo_visual[conds[i]][fp]
        for i in np.where(['no_non80' in c for c in conds])[0]], weights = [0.5, 0.5])
    evo_visualGM['lo_non20'][fp] = mne.combine_evoked([evo_visual[conds[i]][fp]
        for i in np.where(['lo_non20' in c for c in conds])[0]], weights = [0.5, 0.5])
    evo_visualGM['lo_non40'][fp] = mne.combine_evoked([evo_visual[conds[i]][fp]
        for i in np.where(['lo_non40' in c for c in conds])[0]], weights = [0.5, 0.5])
    evo_visualGM['lo_non80'][fp] = mne.combine_evoked([evo_visual[conds[i]][fp]
        for i in np.where(['lo_non80' in c for c in conds])[0]], weights = [0.5, 0.5])
    evo_visualGM['hi_non20'][fp] = mne.combine_evoked([evo_visual[conds[i]][fp]
        for i in np.where(['hi_non20' in c for c in conds])[0]], weights = [0.5, 0.5])
    evo_visualGM['hi_non40'][fp] = mne.combine_evoked([evo_visual[conds[i]][fp]
        for i in np.where(['hi_non40' in c for c in conds])[0]], weights = [0.5, 0.5])
    evo_visualGM['hi_non80'][fp] = mne.combine_evoked([evo_visual[conds[i]][fp]
        for i in np.where(['hi_non80' in c for c in conds])[0]], weights = [0.5, 0.5])
    # when I chose weights = 'equal', the weights were +1 each and the result was a sum of the ERPs

# We have these electrodes:
# keepchannels = ['Nz', 'Fpz', 'Fz', 'FCz', 'Cz', 'CPz', 'Pz', 'Cheek', 'EOG']
# Use Cz, PCz, and Pz for P3
chanpick = ['Cz', 'CPz', 'Pz']
chans = [evo_visualGM['lo_tar20'][13].ch_names.index(c) for c in chanpick]

plt.close(fig=None)
style = dict(show_sensors=False, legend='lower center',
             truncate_xaxis=False, truncate_yaxis=False, ci=None)

# Plot P3 for the three AM separately
for f in range(3):
    #f = 0
    tmp_names = [20,40,80]
    pce({'Low Targets': ERP('lo_tar{}'.format(tmp_names[f]), evo_visualGM),
         'High Targets': ERP('hi_tar{}'.format(tmp_names[f]), evo_visualGM),
         'Low Nontargets': ERP('lo_non{}'.format(tmp_names[f]), evo_visualGM),
         'High Nontargets': ERP('hi_non{}'.format(tmp_names[f]), evo_visualGM),
         #'No load': ERP('no_non{}'.format(tmp_names[f]), evo_visualGM)
         },
         colors={#'No load': 'black',
                 'Low Targets': 'blue', 'Low Nontargets': 'blue',
                 'High Targets': 'red', 'High Nontargets': 'red'},
         linestyles={#'No load': '-',
                     'Low Targets': '-', 'Low Nontargets': ':',
                     'High Targets': '-', 'High Nontargets': ':'},
         **style, title=", ".join(chanpick),
         picks=chans, combine = 'mean') #ylim=dict(eeg=[-5, 11])
    plt.tight_layout()
    # Save figure
    plt.savefig(op.join(dir_results, 'figures', 'figure_visual_ERP_for_{}Hz_N_{}.jpg'.format(tmp_names[f],len(fps))), format='jpg', dpi=600)
    plt.savefig(op.join(dir_results, 'figures', 'figure_visual_ERP_for_{}Hz_N_{}.pdf'.format(tmp_names[f],len(fps))), format='pdf', dpi=600)
    plt.close(fig=None)


# Plot one figure with 3 subplots, highlight 300-400ms
fig, axs = plt.subplots(1, 3, figsize=(20, 7), sharex=True, sharey=True)
tmp_names = [20,40,80]
legg = [False, 'lower center', False]
style = dict(show_sensors=False, #legend='lower center',
             truncate_xaxis=False, truncate_yaxis=False, ci=None)
font = {'family' : 'normal',
        #'weight' : 'normal',
        'size'   : 12}

plt.rc('font', **font)

for leg, am, ax in zip(legg, tmp_names, axs):
    pce({'Low Targets': ERP('lo_tar{}'.format(am), evo_visualGM),
         'High Targets': ERP('hi_tar{}'.format(am), evo_visualGM),
         'Low Nontargets': ERP('lo_non{}'.format(am), evo_visualGM),
         'High Nontargets': ERP('hi_non{}'.format(am), evo_visualGM),
         #'No load': ERP('no_non{}'.format(tmp_names[f]), evo_visualGM)
         },
         colors={#'No load': 'black',
                 'Low Targets': 'blue', 'Low Nontargets': 'blue',
                 'High Targets': 'red', 'High Nontargets': 'red'},
         linestyles={#'No load': '-',
                     'Low Targets': '-', 'Low Nontargets': ':',
                     'High Targets': '-', 'High Nontargets': ':'},
         **style, title='', #{} Hz'.format(am),
         picks=chans, combine = 'mean', legend = leg, axes = ax) #ylim=dict(eeg=[-5, 11])
    ax.axvspan(0.3, 0.4, color='gray', alpha=0.2, lw=0)
    ax.set_title('{} Hz'.format(am))
plt.tight_layout()
plt.savefig(op.join(dir_results, 'figures', 'figure_visual_ERP_all_AM_N_{}.jpg'.format(len(fps))), format='jpg', dpi=600)
plt.savefig(op.join(dir_results, 'figures', 'figure_visual_ERP_all_AM_N_{}.pdf'.format(len(fps))), format='pdf', dpi=600)
plt.close(fig=None)

# =============================================================================
# ----------- EXTRACT MEAN AMPLITUDES -----------------------------------------
# =============================================================================
erp = evo_visual
P3int = np.where((erp[list(erp)[0]][13].times >= 0.300) & (erp[list(erp)[0]][13].times <= 0.400))
chanpick = ['Cz', 'CPz', 'Pz']
chans = [erp[list(erp)[0]][13].ch_names.index(c) for c in chanpick]
x = []
conds = list(erp)
for fp in fps:
    tmp = {'fp': fp}
    for i in conds:
        #tmp['Ntrials_'+i] = erp[i][fp].nave
        tmp['vP3_'+i] = extract_mean_amps(erp, P3int, chans, fp, i)
    x += [tmp]
df = pd.DataFrame(x)
cols = ['fp'] + ['vP3_'+i for i in conds]
#cols = ['fp'] + ['P3_'+i for i in conds] + ['Ntrials_'+i for i in conds]
df = df.reindex(columns=cols)
df.to_csv(op.join(dir_results, 'data_visualP3_N_{}.tsv'.format(len(fps))), sep='\t', index=False)







