import os, mne, sys
import os.path as op
import numpy as np
import pandas as pd
import glob

'''
Compute number of trials for the different conditions (audio and visual)
and save output in csv files.
'''

# Stefan
#dir_main = op.join(op.expanduser("~"),'Documents','Box Sync', 'ASSR2')
# Malina
dir_main = op.join(op.expanduser("~"),'Box Sync', 'ASSR2')
# EEG lab
#dir_main = op.join(op.expanduser("~"), 'Documents', 'Malina_Box','Box Sync', 'ASSR2') # EEG lab

sys.path.append(dir_main + '\\MNE')

mne.set_log_level('WARNING')

# Set file paths
dir_results = op.join(dir_main, 'results')
dir_evo = op.join(dir_main, 'MNE', 'fif', 'evokeds') # contains visual
dir_epo = op.join(dir_main, 'MNE', 'fif', 'epochs') # contains audio

# ========================
# process auditory data
# ========================

# data are saved as epochs
# load filenames
os.chdir(dir_epo)  # change directory to averages folder
files = glob.glob('*audio-epo.fif')

# same condition information as in preprocess file
conds = {'no20_1': 121, 'no20_2': 122,
          'no40_1': 141, 'no40_2': 142,
          'no80_1': 181, 'no80_2': 182,
          'lo20_1': 221, 'lo20_2': 222,
          'lo40_1': 241, 'lo40_2': 242,
          'lo80_1': 281, 'lo80_2': 282,
          'hi20_1': 321, 'hi20_2': 322,
          'hi40_1': 341, 'hi40_2': 342,
          'hi80_1': 381, 'hi80_2': 382}
conds = list(conds)

fps = []

# save number of trials by condition
Danum = {i:[] for i in conds}
for f in files:
    # f = files[-1]
    #print('\n')
    #print(f)
    fp = int(f[2:4])
    fps += [fp]
    epo_fp = mne.read_epochs(f, preload=True)
    for cond in conds:
        #cond = 'no20_1'
        # compute GM (ie evokeds)
        # =========================
        ev = epo_fp[cond].average()
        Danum[cond].append(ev.nave)
print('\n'+'='*80)
print('Audio trials: Mean and minimum number of trials per condition')
print('='*80)
print('Mean:')
print(round(np.mean(pd.DataFrame(Danum), axis=0),1))
print('Min:')
print(np.min(pd.DataFrame(Danum), axis=0))
print('='*80)
Danum['fp'] = fps
Danum = pd.DataFrame(Danum)
cols = Danum.columns.tolist()
Danum = Danum[cols[-1:] + cols[:-1]]
Danum.to_csv(op.join(dir_results, 'data_number_trials_audio.tsv'), sep='\t', index=False)

# ====================
# process visual data
# ====================

# load filenames
os.chdir(dir_evo)  # switch to averages folder
files = glob.glob('*_visual_no_non20_1-ave.fif')

# same condition information as in preprocess file
conds = {'no_non20_1': 121, 'no_non20_2': 122,
                   'no_non40_1': 141, 'no_non40_2': 142,
                   'no_non80_1': 181, 'no_non80_2': 182,
                   'lo_non20_1': 221, 'lo_non20_2': 222,
                   'lo_non40_1': 241, 'lo_non40_2': 242,
                   'lo_non80_1': 281, 'lo_non80_2': 282,
                   'lo_tar20_1': 321, 'lo_tar20_2': 322,
                   'lo_tar40_1': 341, 'lo_tar40_2': 342,
                   'lo_tar80_1': 381, 'lo_tar80_2': 382,
                   'hi_non20_1': 421, 'hi_non20_2': 422,
                   'hi_non80_1': 441, 'hi_non40_2': 442,
                   'hi_non40_1': 481, 'hi_non80_2': 482,
                   'hi_tar20_1': 521, 'hi_tar20_2': 522,
                   'hi_tar40_1': 541, 'hi_tar40_2': 542,
                   'hi_tar80_1': 581, 'hi_tar80_2': 582}
conds = list(conds)
conds.sort()

# save number of trials by condition
Dvnum = {i:[] for i in conds}
fps = []
for f in files:
    # f = files[0]
    fp = int(f[2:4])
    #print('Subject:', fp)
    fps += [fp]
    for cond in conds:
        #cond = 'no_non20_1'
        ev = mne.read_evokeds(op.join(dir_main, 'MNE', 'fif', 'evokeds',
                                      'fp%02d_visual_' % fp + cond + '-ave.fif' ), condition = cond)
        Dvnum[cond].append(ev.nave)
Dvnum = pd.DataFrame(Dvnum)
print('\n'+'='*80)
print('Visual trials: Mean and minimum number of trials per condition')
print('='*80)
print('Mean:')
print(round(np.mean(pd.DataFrame(Dvnum), axis=0),1))
print('Min:')
print(np.min(pd.DataFrame(Dvnum), axis=0))
print('='*80)
Dvnum['fp'] = fps
Dvnum = pd.DataFrame(Dvnum)
cols = Dvnum.columns.tolist()
Dvnum = Dvnum[cols[-1:] + cols[:-1]]
Dvnum.to_csv(op.join(dir_results, 'data_number_trials_visualP3.csv'), sep='\t', index=False)