import os, mne, sys
import os.path as op
import numpy as np
import pandas as pd

# Stefan
#dir_main = op.join(op.expanduser("~"),'Documents','Box Sync', 'ASSR_study2')
# Malina
dir_main = op.join(op.expanduser("~"),'Box Sync', 'ASSR2')
# EEG lab
#dir_main = op.join(op.expanduser("~"), 'Documents', 'Malina_Box','Box Sync', 'ASSR2') # EEG lab

sys.path.append(dir_main + '\\MNE')
from rejectvisual import rejectvisual

#mne.set_log_level('WARNING')

os.chdir(dir_main)

# For some subjects, the diodes glitched and the visual ERPs could not be identified
fps_novisualERPs = [2] #[1,2,3,4,5,6,7,8,9,10,11,12]
fps_100Hz = [1,2,3,4]
fps_oldrand = [1,2,3,4,5,6,7,8,9,10] # For these subjects, the audio stim inversion alternated as follows: [0,0,0,1,1,1,0,0,0,1,1,1,0,0,0,1,1,1]
fps_1024Hz = [1,2,3]

# Fp 1 misses 11 photodiodes (4435 instead of 4446)

tmpdiag = [] #for diagnostics
def preprocess(fp):
    #fp = 35
    print('\n'+'='*50)
    print('Subject:', fp)
    print('='*50+'\n')
    # Input files
    bdffile = op.join(dir_main, 'bdf', 'assr2_fp%02d.bdf' % fp)
    logfile = op.join(dir_main, 'logfiles', 'fp_%d.txt' % fp)
    #bad_channels_file = op.join(dir_main, 'MNE', 'fif', 'bad_channels', 'fp%02d.txt' % fp)
    # Output files
    #evofile_visual = op.join(dir_main, 'MNE', 'fif', 'evokeds', 'fp%02dvisual-ave.fif' % fp)
    #evofile_audio = op.join(dir_main, 'MNE', 'fif', 'evokeds', 'fp%02daudio-ave.fif' % fp)
    epofile_audio = op.join(dir_main, 'MNE', 'fif', 'epochs', 'fp%02daudio-epo.fif' % fp)
    epofile_audio_res = op.join(dir_main, 'MNE', 'fif', 'epochs', 'fp%02daudio_resampl-epo.fif' % fp)
    epofile_audio_minute = op.join(dir_main, 'MNE', 'fif', 'epochs', 'fp%02daudio-minute-epo.fif' % fp)
    epofile_audio_minute_res = op.join(dir_main, 'MNE', 'fif', 'epochs', 'fp%02daudio-minute_resampl-epo.fif' % fp)
    bad_trials_file_audio = op.join(dir_main, 'MNE', 'fif', 'bad_trials', 'fp%02daudio.txt' % fp)
    bad_trials_file_visual = op.join(dir_main, 'MNE', 'fif', 'bad_trials', 'fp%02dvisual.txt' % fp)
    tmpd = [fp] # diagnostics for individual

    # read bdf file
    if fp in [18,26]: # these subject have two files
        bdffile = op.join(dir_main, 'bdf', 'assr2_fp%02d_a.bdf' % fp)
        raw = mne.io.read_raw_bdf(bdffile, preload=True)
        bdffile = op.join(dir_main, 'bdf', 'assr2_fp%02d_b.bdf' % fp)
        raw2 = mne.io.read_raw_bdf(bdffile, preload=True)
        raw = mne.concatenate_raws([raw, raw2])
        del(raw2)
    else:
        raw = mne.io.read_raw_bdf(bdffile, preload=True)

    # Apply 0.1 Hz high pass filter (butterworth 4th two-pass)
    #mne.io.Raw.filter(raw, l_freq=1, h_freq=None, method='iir', iir_params=None)
    mne.io.Raw.filter(raw, l_freq=0.1, h_freq=None, method='iir', iir_params=None)
    #mne.io.Raw.filter(raw, l_freq=0.1, h_freq=None, method='iir',
    #                  iir_params = dict(order = 2, ftype = 'butter'))
    # If you run the code above, the warning that shows up in the console says:
    #IIR filter parameters
    #---------------------
    #Butterworth highpass zero-phase (two-pass forward and reverse) non-causal filter:
    #- Filter order 4 (effective, after forward-backward)
    #- Cutoff at 0.10 Hz: -6.02 dB
    # So although the order specified is 2, it shows up as 4.
    #and if you specify the order to be 4, then it shows up as 8.

    # set nose to reference
    raw.set_eeg_reference(ref_channels=['Nz'])

    # create eog (Fpz anode = +, cheek cathode = -) and keep Fpz and Cheek
    raw = mne.set_bipolar_reference(raw, anode=['Fpz'], cathode=['Cheek'],
                                    ch_name='EOG', drop_refs=False)

    Fs = raw.info['sfreq']

    # read status channel
    # ======================
    events = mne.find_events(raw=raw, stim_channel='Status', min_duration=0.001)

    # fp 15 has too many blocks. recording was started again after 2 blocks. remove 2 first blocks.
    if fp == 15:
        events = events[992:len(events),:] # first block code

    # For fp35, after block 11 the experiment was force-stopped. 
    # I started it from block 12, first time to see if it would work, 
    # second time I realized that I have to change the logfiles names 
    # so that it doesn't overwrite. So the thrid time it started it was the 
    # real experiment. Block 12 is 40Hz no load, that is why the number of 
    # trials is slightly not correct. I will remove samples from 5456 to 5463. 
    # The real block 12 starts from sample 5464.  
    if fp == 35:
        events = np.delete(events, range(5456,5464), 0)


    # recode decimal to binary
    # StimTracker output is organized as follows:
    # Lightsensor  1 Trig. 1 / 1 (LSB)
    # Lightsensor 2 Trig. 2 / 2
    # Lightsensor 3 Trig. 3 / 4
    # Lightsensor 4 Trig. 4 / 8
    # Audio, left channel Trig. 5 / 16
    # Audio, right channel Trig. 6 / 32
    # Microphone Trig. 7 / 64
    # Event marker bit 0 Trig. 8 / 128
    # Event marker bit 1 Trig. 9 / 1 (MSB)
    # Event marker bit 2 Trig. 10 / 2
    # TTL input, line 1 Trig. 11 / 4
    # TTL input, line 2 Trig. 12 / 8
    # TTL input, line 3 Trig. 13 / 16
    # TTL input, line 4 Trig. 14 / 32
    # TTL input, line 5 Trig. 15 / 64
    # TTL input, line 6 Trig. 16 / 128
    bincodes=[(bin(x)[2:])[::-1] for x in events[:,2]]
    bincodes = [x + '0'*(16-len(x)) for x in bincodes]
    # bincodes organized from left to right to match StimTracker

    # extract TTL input
    ttls = [int(x[10:]) for x in bincodes]
    ttls = np.array(ttls)

    # extract info about block onsets and load
    # ========================================
    events = np.hstack((events, np.zeros((events.shape[0],1), dtype = 'int')))
    conds_visual = [100000, 10000, 10001, 1000, 1001, 11111, 101111, 110111]
#    if fp in fps_100Hz:
#        event_id_visual = ['NoLoad', # refers to all trials because there are no targets
#                           'LowNonTarget',
#                           'LowTarget',
#                           'HiNonTarget',
#                           'HiTarget',
#                           'BlockOnset_20',
#                           'BlockOnset_40',
#                           'BlockOnset_100']
#    else:

    event_id_visual = ['NoLoad', # refers to all trials because there are no targets
                       'LowNonTarget',
                       'LowTarget',
                       'HiNonTarget',
                       'HiTarget',
                       'BlockOnset_20',
                       'BlockOnset_40',
                       'BlockOnset_80']
    #each block has 240 trials (20% targets) plus 7 nontargets at the beginning.
    #Count
    #NoLoad = 1482
    #LowNonTarget = 1194 ((192+7) * 4)
    #LowTarget = 288
    #HighNonTarget = 1194
    #HighTarget = 288
    #BlockOnset = 3 x 6 (3 different sounds, 6 blocks each)
    print('\n'+'='*50)
    print('Visual events per condition:')
    print('='*50)
    cnt = []
    for i in range(len(conds_visual)):
        indx = np.where(ttls == conds_visual[i])
        print(event_id_visual[i] + ' = {}'.format(np.size(indx)))
        events[indx,3] = (i+1)*10
        cnt.append(np.size(indx))
        # conditions are coded as 10 thru 80, as listed in event_id_visual
    if not(np.any(fp in fps_novisualERPs+[19])):
        # fp 19 has 1470 no load trials
        if not cnt == [1482, 1194, 288, 1194, 288, 6, 6, 6]:
            raise Warning("Number of visual trials is off.")
    tmpd.append(cnt)


    # ========================
    # Process auditory data
    # ========================

    # auditory onsets
    # ================
    # audio is in column 5 and 6
    audioonL = np.array([int(x[4]) for x in bincodes])
    audioonR = np.array([int(x[5]) for x in bincodes])
    print('\n')
    if np.array_equal(audioonL, audioonR):
        print("Data check: Audio L and R are equal")
    else:
        #raise Warning("Audio L and R differ from each other")
        print("Warning: Audio L and R are not equal.")
        #print("Onsets are defined as both are on.") this worked in study 1
        #but not here because either diode did not work for some subjects.
        print("Onsets are defined as either one is on.")
    events = np.column_stack((events,np.logical_or(audioonL == 1, audioonR == 1)))

    # block onsets (check for subsequent audio onset)
    onsblock = np.where(np.any((events[:,3]==60, events[:,3]==70, events[:,3]==80), axis=0))[0]
    onsaudio = []
    for i in onsblock:
        tmp=events[i:i+10,4] #exstract audio onsets
        onsaudio.append(np.argmax(tmp==1))
    #if (min(onsaudio) == 0 or max(onsaudio) > 2):
        #raise Warning("Block onsets are not followed by audio (or within 2 samples)")
#    if fp == 24: #fix that block 4 had no apparent audio delay
#        onsaudio[4] = 2
    # create the audio onset
    onsaudio = onsblock + onsaudio # create the audio onset
    onsdelay = events[onsaudio,0] - events[onsblock,0] # Check the delay between the block and audio onsets
    print('\n'+'='*50)
    print('Auditory block onsets: Delay between block onset and audio onset.')
    print('='*50)
    print('Min  = {} ms'.format(round(np.min(onsdelay)/Fs*1000,2)))
    print('Max  = {} ms'.format(round(np.max(onsdelay)/Fs*1000,2)))
    print('Mean = {} ms'.format(round(np.mean(onsdelay)/Fs*1000,2)))
    print('='*50)
    tmpd.append([int(np.min(onsdelay)/Fs*1000), int(np.max(onsdelay)/Fs*1000)])

    blockint = np.diff(events[onsaudio,0])/Fs # interval between blocks in sec
    # save block length as diagnostics
    tmpd.append([int(np.min(blockint)), int(np.max(blockint))])

    # move block onset to audio onset
    onsaudio = events[onsaudio,0]

    # chop up 2 min after block onset into shorter epochs
    lenepoch = 1.5625 # epoch length in sec
    nepoch = int(round(120 / lenepoch,0))
    # Below is to have an even number of epochs
#    import math
#    nepoch = int(math.floor(120/lenepoch))
    lenepochtf = int(lenepoch * Fs)
    audioepochons = []
    for i in range(0,len(onsaudio)):
        audioepochons = audioepochons + [onsaudio[i] + x * lenepochtf for x in range(nepoch)]
    audioepochons = np.asarray(audioepochons)

    # conditions are randomized within a set of three (no, lo, hi)
    # determine condition for each block (from actual next trials)
    evtmp = events[events[:,3] > 0, 3]
    blockLoad = evtmp[np.asarray(np.where(np.any((evtmp==60, evtmp==70, evtmp==80), axis = 0))) + 1][0]
    blockLoad[np.where(blockLoad == 10)] = 100 # no load
    blockLoad[np.where(np.logical_and(blockLoad >= 20, blockLoad <= 30))] = 200 # low load
    blockLoad[np.where(np.logical_and(blockLoad >= 40, blockLoad <= 50))] = 300 # hi load
    blockcnt = np.unique(blockLoad, return_counts=True)
    # check that there are 6 blocks each
    if (np.array_equal(blockcnt[0], [100,200,300]) == False) or (np.array_equal(blockcnt[1], [6,6,6]) == False):
        print("Blocks and their frequency:")
        print(blockcnt[0])
        print(blockcnt[1])
        #raise Warning("Not equal amount of load levels")
    # combine block (10, 20, 30) and condition (1 to 3)

    # determine audio for each block (from actual trials)
    blockAudio = evtmp[np.asarray(np.where(np.any((evtmp==60, evtmp==70, evtmp==80), axis = 0)))][0]
    blockAudio[np.where(blockAudio == 60)] = 20 # 20 Hz
    blockAudio[np.where(blockAudio == 70)] = 40 # 40 Hz
    blockAudio[np.where(blockAudio == 80)] = 80 # 100 Hz or 80 Hz
#    if fp in fps_100Hz:
#        blockAudio[np.where(blockAudio == 80)] = 10 # 100 Hz
#    else:
#        blockAudio[np.where(blockAudio == 80)] = 80 # 80 Hz
    blockcnt = np.unique(blockAudio, return_counts=True)
    # check that there are 6 blocks each
    if (np.array_equal(blockcnt[0], [20,40,80]) == False) or (np.array_equal(blockcnt[1], [6,6,6]) == False):
        print("Blocks and their frequency:")
        print(blockcnt[0])
        print(blockcnt[1])
        #raise Warning("Not equal amount of audio stimuli")

    # determine load-audio combination for each block (from actual trials)
    blockLoadAudio = blockLoad + blockAudio
    blockcnt = np.unique(blockLoadAudio, return_counts=True)
    # check that there are 6 blocks each
    if (np.array_equal(blockcnt[0], [120, 140, 180, 220, 240, 280, 320, 340, 380]) == False) or (np.array_equal(blockcnt[1], [2,2,2,2,2,2,2,2,2]) == False):
        print("Blocks and their frequency:")
        print(blockcnt[0])
        print(blockcnt[1])
        #raise Warning("Not equal amount of audio - load combinations")

#    #Divide condition coding into 1 minute blocks
#    audioepochblocks = np.add(np.repeat(blockLoad,nepoch), np.hstack([np.repeat(i*10, nepoch*9) for i in range(1,3)]))
#
#    #BELOW IS DIVIDING IT BY MINUTE NOT BLOCKS!! WRONG------
#    len(np.hstack([np.hstack([np.repeat(10, 38), np.repeat(20, 39)]) for i in range(18)]))
#    audioepochblocks = np.add(np.repeat(blockLoadAudio,nepoch), np.hstack([np.hstack([np.repeat(10, 38), np.repeat(20, 39)]) for i in range(len(onsaudio))]))
#    ------------

    # add block number:
    if fp in fps_oldrand:
        audioepochblocks = np.add(np.repeat(blockLoadAudio,nepoch), np.hstack([np.hstack([np.repeat(i, nepoch*3) for i in range(1,3)]) for i in range(3)]))
    else:
        audioepochblocks = np.add(np.repeat(blockLoadAudio,nepoch), np.hstack([np.repeat(i, nepoch*9) for i in range(1,3)]))

# TESTING 80 Hz
#    if fp == 9999:
#        audioepochblocks = np.add(np.repeat(blockLoadAudio,nepoch), np.hstack([np.repeat(i, nepoch*3) for i in range(1,3)]))

    blockcnt = np.unique(audioepochblocks, return_counts=True)
    if (np.array_equal(blockcnt[0], [121,122,141,142,181,182,221,222,241,242,281,282,321,322,341,342,381,382]) == False) or (np.array_equal(blockcnt[1], [77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77]) == False):
        print("Blocks and their frequency:")
        print(blockcnt[0])
        print(blockcnt[1])

    events_audio = np.column_stack((audioepochons, np.zeros(len(audioepochons)), audioepochblocks)).astype('int')
    # auditory epochs
    # ================

#    if fp in fps_100Hz:
#        event_id_audio = {'no20_1': 121, 'no20_2': 122,
#                          'no40_1': 141, 'no40_2': 142,
#                          'no100_1': 181, 'no100_2': 182,
#                          'lo20_1': 221, 'lo20_2': 222,
#                          'lo40_1': 241, 'lo40_2': 242,
#                          'lo100_1': 281, 'lo100_2': 282,
#                          'hi20_1': 321, 'hi20_2': 322,
#                          'hi40_1': 341, 'hi40_2': 342,
#                          'hi100_1': 381, 'hi100_2': 382}
#    else:
    event_id_audio = {'no20_1': 121, 'no20_2': 122,
                      'no40_1': 141, 'no40_2': 142,
                      'no80_1': 181, 'no80_2': 182,
                      'lo20_1': 221, 'lo20_2': 222,
                      'lo40_1': 241, 'lo40_2': 242,
                      'lo80_1': 281, 'lo80_2': 282,
                      'hi20_1': 321, 'hi20_2': 322,
                      'hi40_1': 341, 'hi40_2': 342,
                      'hi80_1': 381, 'hi80_2': 382}

# FOR 80 Hz TEST!
#    event_id_audio = {'no20_1': 121, 'no80_2': 182,
#                      'lo20_2': 222, 'lo40_1': 241,
#                      'hi40_2': 342, 'hi80_1': 381}

    # keep all channels (see raw.ch_names)
    keepchannels = ['Nz', 'Fpz', 'Fz', 'FCz', 'Cz', 'CPz', 'Pz', 'Cheek', 'EOG']
    picks = mne.pick_types(raw.info, eeg=True, meg=False, eog=True,
                           selection=keepchannels)
    if fp in fps_1024Hz:
        epochs_params = dict(events=events_audio, event_id = event_id_audio, tmin=0.000976562, tmax=lenepoch,
                             baseline= (None), picks=picks)
    else:
        epochs_params = dict(events=events_audio, event_id = event_id_audio, tmin=0.000488281, tmax=lenepoch,
                         baseline= (None), picks=picks)
    # baseline = whole epoch to get rid of any overall level differences
    # In fact, if you do not use a high-pass filter att all and there is no baseline,
    # the electrode levels differ a lot from each other! Rereferencing does not help with that.
    #
    # If tmin starts at 0, there are 1601 tfs instead of only 1600
    # because it includes time 0. This messes up the frequency analysis
    epochs_audio = mne.Epochs(raw, **epochs_params, preload=True)

    # Upsample the ones that were recorded at 1024 Hz
    # Upsample to 2048 Hz
    if fp in fps_1024Hz:
        print('Original sampling rate:', epochs_audio.info['sfreq'], 'Hz')
        epochs_audio_resampled = epochs_audio.copy().resample(2048, npad='auto')
        print('New sampling rate:', epochs_audio_resampled.info['sfreq'], 'Hz')

    # alternative analysis:
    # ====================
    # consider development within a block
    # compute for each 1-min section with a block
    # 100, 200 etc codes for block 1,2,3,4
    # 10, 20, 30, 40 codes for load
    # 1, 2, 3 codes for minute within a block
    events_audio[:,2] = events_audio[:,2]*10 #just multiply by 10
    events_audio[:,2] = np.add(events_audio[:,2],
                               np.tile(np.hstack((np.repeat(1,39), np.repeat(2,38))), len(blockLoad)))
    blockcnt = np.unique(events_audio[:,2], return_counts=True)

    # code for load (no, lo, hi), audio (20, 40, 80 or 100) block (1 to 2), and minute (m1 to m3)
#    if fp in fps_100Hz:
#        event_id_audio_minute = {'no20_1_1m': 1211, 'no20_1_2m': 1212, 'no20_2_1m': 1221, 'no20_2_2m': 1222,
#                                  'no40_1_1m': 1411, 'no40_1_2m': 1412, 'no40_2_1m': 1421, 'no40_2_2m': 1422,
#                                  'no100_1_1m': 1811, 'no100_1_2m': 1812, 'no100_2_1m': 1821, 'no100_2_2m': 1822,
#                                  'lo20_1_1m': 2211, 'lo20_1_2m': 2212, 'lo20_2_1m': 2221, 'lo20_2_2m': 2222,
#                                  'lo40_1_1m': 2411, 'lo40_1_2m': 2412, 'lo40_2_1m': 2421, 'lo40_2_2m': 2422,
#                                  'lo100_1_1m': 2811, 'lo100_1_2m': 2812, 'lo100_2_1m': 2821, 'lo100_2_2m': 2822,
#                                  'hi20_1_1m': 3211, 'hi20_1_2m': 3212, 'hi20_2_1m': 3221, 'hi20_2_2m': 3222,
#                                  'hi40_1_1m': 3411, 'hi40_1_2m': 3412, 'hi40_2_1m': 3421, 'hi40_2_2m': 3422,
#                                  'hi100_1_1m': 3811, 'hi100_1_2m': 3812, 'hi100_2_1m': 3821, 'hi100_2_2m': 3822}
#    else:
    event_id_audio_minute = {'no20_1_1m': 1211, 'no20_1_2m': 1212, 'no20_2_1m': 1221, 'no20_2_2m': 1222,
                              'no40_1_1m': 1411, 'no40_1_2m': 1412, 'no40_2_1m': 1421, 'no40_2_2m': 1422,
                              'no80_1_1m': 1811, 'no80_1_2m': 1812, 'no80_2_1m': 1821, 'no80_2_2m': 1822,
                              'lo20_1_1m': 2211, 'lo20_1_2m': 2212, 'lo20_2_1m': 2221, 'lo20_2_2m': 2222,
                              'lo40_1_1m': 2411, 'lo40_1_2m': 2412, 'lo40_2_1m': 2421, 'lo40_2_2m': 2422,
                              'lo80_1_1m': 2811, 'lo80_1_2m': 2812, 'lo80_2_1m': 2821, 'lo80_2_2m': 2822,
                              'hi20_1_1m': 3211, 'hi20_1_2m': 3212, 'hi20_2_1m': 3221, 'hi20_2_2m': 3222,
                              'hi40_1_1m': 3411, 'hi40_1_2m': 3412, 'hi40_2_1m': 3421, 'hi40_2_2m': 3422,
                              'hi80_1_1m': 3811, 'hi80_1_2m': 3812, 'hi80_2_1m': 3821, 'hi80_2_2m': 3822}

    if fp in fps_1024Hz:
        epochs_params = dict(events=events_audio, event_id = event_id_audio_minute, tmin=0.000976562, tmax=lenepoch,
                             baseline= (None), picks=picks)
    else:
        epochs_params = dict(events=events_audio, event_id = event_id_audio_minute, tmin=0.000488281, tmax=lenepoch,
                             baseline= (None), picks=picks)
    epochs_audio_minute = mne.Epochs(raw, **epochs_params, preload=True)

    # Upsample the ones that were recorded at 1024 Hz
    # Upsample to 2048 Hz
    if fp in fps_1024Hz:
        print('Original sampling rate:', epochs_audio_minute.info['sfreq'], 'Hz')
        epochs_audio_minute_resampled = epochs_audio_minute.copy().resample(2048, npad='auto')
        print('New sampling rate:', epochs_audio_minute_resampled.info['sfreq'], 'Hz')


    # ========================
    # Process visual data
    # ========================
    # match visual onsets to photodiode 1
    # matching is slightly different from study 1 because here,
    # at least one subject had too many photodiode events
    if (fp in fps_novisualERPs):
        tmpd.append([-99, -99, -99])
    else:
        photodiode = np.array([int(x[0]) for x in bincodes])
        events = np.column_stack((events, photodiode))
        onsconds = np.where(np.logical_and(events[:,3] > 0, events[:,3]<60))[0]
        if np.sum(photodiode) != 4446:
            print('Wrong number of photodiode onsets! {} instead of 4446'.format(np.sum(photodiode)))
        if fp == 1:
            onsphoto = photodiode[np.asarray(onsconds)-1]
            if np.any(onsphoto!=1):
                print("Some visual onsets do not have an immediate photodiode")
                #raise Warning("Some visual onsets do not have an immediate photodiode")
            onsphoto = np.asarray(onsconds)-onsphoto
        else:
            onsphoto = photodiode[np.asarray(onsconds)+1]
            if np.any(onsphoto!=1):
                raise Warning("Some visual onsets do not have an immediate photodiode")
            onsphoto = np.asarray(onsconds)+1
        onsdelay = events[onsphoto,0] - events[onsconds,0]
        print('\n'+'='*50)
        print('Visual events: delay between condition trigger and photodiode.')
        print('='*50)
        print('Min  = {} ms'.format(round(np.min(onsdelay)/Fs*1000,2)))
        print('Max  = {} ms'.format(round(np.max(onsdelay)/Fs*1000,2)))
        print('Mean = {} ms'.format(round(np.mean(onsdelay)/Fs*1000,2)))
        print('='*50)
        tmpd.append([int(np.min(onsdelay)/Fs*1000), int(np.mean(onsdelay)/Fs*1000), int(np.max(onsdelay)/Fs*1000)])
        events_visual = events[onsphoto,0:3]
        events_visual[:,2] = events[onsconds,3]*10
        # condition is 10 to 50
        # add block number as 1 thru 2
        # 247*18 = 4446 trials
        # add block number:
        if fp in fps_oldrand:
            blockindx = np.hstack([np.hstack([np.tile(np.hstack((np.repeat(0,7), np.repeat(i,240))),3) for i in range(1,3)]) for i in range(3)])
            logindx = np.hstack([np.hstack([np.tile(np.repeat(i,240),3) for i in range(1,3)]) for i in range(3)])
            ampindx = np.hstack([np.repeat(i, 240) for i in blockAudio])
        else:
            blockindx = np.hstack([np.tile(np.hstack((np.repeat(0,7), np.repeat(i,240))),9) for i in range(1,3)])
            logindx = np.hstack([np.tile(np.repeat(i,240),9) for i in range(1,3)])
            ampindx = np.hstack([np.repeat(i, 240) for i in blockAudio])
        events_visual = np.column_stack((events_visual,blockindx))
        # remove the first seven trials in each block
        events_visual = np.delete(events_visual,np.where(blockindx==0), axis=0) #axis 0 deletes rows
        events_visual[:,2] = np.add(events_visual[:,2],events_visual[:,3])
       # add info about the sound
        events_visual = np.column_stack((events_visual,ampindx))
        events_visual[:,2] = np.add(events_visual[:,2],events_visual[:,4])
        events_visual = events_visual[:,0:3]


        # event_id_visual = {'no_non1': 101, 'no_non2': 102,
        #                    'lo_non1': 201, 'lo_non2': 202,
        #                    'lo_tar1': 301, 'lo_tar2': 302,
        #                    'hi_non1': 401, 'hi_non2': 402,
        #                    'hi_tar1': 501, 'hi_tar2': 502}

        # dictionary of visual conditions
        event_id_visual = {'no_non20_1': 121, 'no_non20_2': 122,
                           'no_non40_1': 141, 'no_non40_2': 142,
                           'no_non80_1': 181, 'no_non80_2': 182,
                           'lo_non20_1': 221, 'lo_non20_2': 222,
                           'lo_non40_1': 241, 'lo_non40_2': 242,
                           'lo_non80_1': 281, 'lo_non80_2': 282,
                           'lo_tar20_1': 321, 'lo_tar20_2': 322,
                           'lo_tar40_1': 341, 'lo_tar40_2': 342,
                           'lo_tar80_1': 381, 'lo_tar80_2': 382,
                           'hi_non20_1': 421, 'hi_non20_2': 422,
                           'hi_non80_1': 441, 'hi_non40_2': 442,
                           'hi_non40_1': 481, 'hi_non80_2': 482,
                           'hi_tar20_1': 521, 'hi_tar20_2': 522,
                           'hi_tar40_1': 541, 'hi_tar40_2': 542,
                           'hi_tar80_1': 581, 'hi_tar80_2': 582}
        evcnt = np.unique(events_visual[:,2],return_counts=True)
        if (np.array_equal(evcnt[0], [121, 122, 141, 142, 181, 182,
                                      221, 222, 241, 242, 281, 282,
                                      321, 322, 341, 342, 381, 382,
                                      421, 422, 441, 442, 481, 482,
                                      521, 522, 541, 542, 581, 582]) == False) or (np.array_equal(evcnt[1], [240, 240, 240, 240, 240, 240,
                                                                                                             192, 192, 192, 192, 192, 192,
                                                                                                             48, 48, 48, 48, 48, 48,
                                                                                                             192, 192, 192, 192, 192, 192,
                                                                                                             48, 48, 48, 48, 48, 48]) == False):
            print("Condition codes and their frequency:")
            print(evcnt[0])
            print(evcnt[1])

        # visual epochs
        # ==============
        epochs_params = dict(events=events_visual, event_id=event_id_visual,
                             tmin=-.1, tmax=.5, # -100 to +500 ms are preregistered
                             baseline=(None, 0.0), picks=picks)
        epochs_visual = mne.Epochs(raw, **epochs_params, preload=True)
        epochs_visual.resample(256, npad='auto')

        # read logfile (of visual events)
        # ===============================
        logd = pd.read_csv(logfile, sep = '\t')
        # delete first 7 standards
        logd = logd[logd['trial_nr'] > 7]
        # select relevant columns
        logd = logd[['block_nr','tone','load','target']]
        # reconstruct codes
        logd['load'] = logd['load'].replace(to_replace = {'no': 10,
                                                          'low': 20,
                                                          'high': 40})
        if fp in fps_100Hz:
            logd['tone'] = logd['tone'].replace(to_replace = {'20Hz_AM': 20, '20Hz_AM_inv': 20,
                                                              '40Hz_AM': 40, '40Hz_AM_inv': 40,
                                                              '100Hz_AM': 80, '100Hz_AM_inv': 80,})
        else:
            logd['tone'] = logd['tone'].replace(to_replace = {'20Hz_AM': 20, '20Hz_AM_inv': 20,
                                                              '40Hz_AM': 40, '40Hz_AM_inv': 40,
                                                              '80Hz_AM': 80, '80Hz_AM_inv': 80,})
        logd = logd.values
        logd[logd[:,3] == 1, 2] += 10
        logd[:,2] *= 10
        logd[:,2] = logd[:,2] + logd[:,1] + logindx
        if np.any(logd[:,1] != np.hstack(([np.repeat(i, 240) for i in blockAudio]))):
            raise Warning('The auditory codes from the EEG file do not match the logfile!')
        if np.any(logd[:,2]!=events_visual[:,2]):
            raise Warning('The visual codes from the EEG file do not match the logfile!')

    # Reject visual
    def cleandata(file, epochs, timeon, timeoff, info):
        if op.isfile(file):
            if os.stat(file).st_size != 0:
                with open(file, 'r') as f:
                    bad_trials = [list(map(int, data.split(','))) for data in f][0]
            else: bad_trials = []
        else:
        #        inp = 'n'
        #        while inp == 'n':
            bad_trials = rejectvisual(epochs, fp, timeon, timeoff, info)
        #    mne.viz.plot_evoked_topo(epochs.average(), title= 'Fp: %s' % fp,
        #                             color=['black'])
        #            inp = input('\nInput n to reset.\nPress enter to continue: ')
        #            if inp == 'n':
        #                print('Resetting.')
        #            else:
        #                print('Removing %i bad trials\n' % len(bad_trials))
            with open(file, 'w') as f:
                f.write(','.join(str(t) for t in bad_trials))
        epochs.drop(bad_trials)

    # remove noisy trials
    print('Clean audio epochs')
    cleandata(bad_trials_file_audio, epochs_audio, 0, lenepoch, "Audio")
    # because the two audio analyses use the same data, just reject the same trials
    cleandata(bad_trials_file_audio, epochs_audio_minute, 0, lenepoch, "Audio")

    if fp in fps_1024Hz:
        # also clean the resampled data
        cleandata(bad_trials_file_audio, epochs_audio_resampled, 0, lenepoch, "Audio")
        # because the two audio analyses use the same data, just reject the same trials
        cleandata(bad_trials_file_audio, epochs_audio_minute_resampled, 0, lenepoch, "Audio")

    if not(fp in fps_novisualERPs):
        print('Clean visual epochs')
        cleandata(bad_trials_file_visual, epochs_visual, -0.1, 0.5, "Visual")

    # save epoched audio
    print("Save audio epochs:")
    for c in list(event_id_audio):
        print(c + " = " + str(len(epochs_audio[c])) + " trials")
    epochs_audio.save(epofile_audio, overwrite=True)



    print("Save audio epochs per minute:")
    for c in list(event_id_audio_minute):
        print(c + " = " + str(len(epochs_audio_minute[c])) + " trials")
    epochs_audio_minute.save(epofile_audio_minute, overwrite=True)

    if fp in fps_1024Hz:
        epochs_audio_minute_resampled.save(epofile_audio_minute_res, overwrite=True)
        epochs_audio_resampled.save(epofile_audio_res, overwrite=True)

    # save evoked visual
    if not(fp in fps_novisualERPs):
        conds = list(event_id_visual)
        evos = {}
        print("Save visual evokeds:")
        for cond in conds:
            ev = epochs_visual[cond].average()
            print(cond + " = " + str(ev.nave) + " trials")
            evos[cond] = ev
            ev.save(op.join(dir_main, 'MNE', 'fif', 'evokeds',
                            'fp%02d_visual_' % fp + cond + '-ave.fif' ))
        #We could not write evokeds because nave was messed up in mne19.
        #It used the nave info from first condition for all conditions.
        #mne.write_evokeds(evofile_visual, [evos[cond] for cond in conds])
    tmpd = np.hstack(tmpd)
    tmpdiag.append(tmpd)
    print('\n'+'='*50)
    print('Preprocessing done for subject', fp)
    print('='*50)

x = int(input('Start from: '))
fps = np.array(range(x,61)) # last subject is 47
#fps = np.delete(fps, np.where(np.isin(fps,[35, 38]))) #fp 35 + 38 lacked data
for fp in fps:
    preprocess(fp)

#tmpdiag = np.row_stack(tmpdiag)
#df = pd.DataFrame(tmpdiag, columns = ["fp", "N_no",
#                                     "N_lo_nontar", "N_lo_tar",
#                                     "N_md_nontar", "N_md_tar",
#                                     "N_hi_nontar", "N_hi_tar",
#                                     "N_blocks",
#                                     "DelayAudio_ms_min", "DelayAudio_ms_max",
#                                     "IntBlocks_sec_min", "IntBlocks_sec_max",
#                                     "DelayPhoto_ms_min", "DelayPhoto_ms_mean", "DelayPhoto_ms_max"])
#tmp = np.min(df["IntBlocks_sec_min"])
#print('Minimum interval between blocks (in sec):', tmp)
#if tmp > 184:
#    print('Congrats: No subject had an interval between blocks that was shorter than the minimum (184 s)')
#else:
#    print('Ouch: At least one subject had an interval between blocks that was shorter than the minimum (184 s)')
#df.to_csv('Diode_diagnostics.csv', sep='\t', index=False)