import numpy as np
from matplotlib.widgets import RectangleSelector
import matplotlib.pyplot as plt


class Highlighter(object):
    '''
    Credit to Joe Kington:
    https://stackoverflow.com/a/31926249
    '''
    def __init__(self, ax, x, y):
        self.ax = ax
        self.canvas = ax.figure.canvas
        self.x, self.y = x, y
        self.mask = np.zeros(x.shape, dtype=bool)
        self._highlight = ax.scatter([], [], s=2, color='yellow', zorder=10)
        self.selector = RectangleSelector(ax, self, useblit=True)

    def __call__(self, event1, event2):
        self.mask |= self.inside(event1, event2)
        xy = np.column_stack([self.x[self.mask], self.y[self.mask]])
        self._highlight.set_offsets(xy)
        self.canvas.draw()

    def inside(self, event1, event2):
        """Returns a boolean mask of the points inside the rectangle defined by
        event1 and event2."""
        # Note: Could use points_inside_poly, as well
        x0, x1 = sorted([event1.xdata, event2.xdata])
        y0, y1 = sorted([event1.ydata, event2.ydata])
        mask = ((self.x > x0) & (self.x < x1) &
                (self.y > y0) & (self.y < y1))
        return mask

# start, stop = -0.1, 0.6
def rejectvisual(epochs, fp, start, stop, info):
    # Clean bad trials
    # channel indicies
    chIndx = np.array([epochs.ch_names.index(ch)
                      for ch in epochs.ch_names])
    # Trial indicies
    trIndx = np.array(range(len(epochs.events)))

    bad_trials = []
    selected_trials = [1]
    selected_channels = [1]
    # loop while trials or channels are selected
    while len(selected_trials) != 0 or len(selected_channels) != 0:
        subset = np.where((epochs.times >= start) & (epochs.times <= stop))
        # Extract ranges in uV (list with epochs containing lists with
        # channels) only for selected channels
        rng_ch = [[np.ptp(ch[subset]) for ch in tr[chIndx]]
                  for tr in epochs[trIndx]]

        rng_heat = []
        for i, tr in enumerate(epochs):
            if i in trIndx:
                rng_heat += [[np.ptp(ch[subset]) for ch in tr]]
            else:
                rng_heat += [len(epochs.ch_names)*[0]]

# =============================================================================
#             rng_heat = [[np.ptp(ch) for ch in tr] if i in trIndx
#                         else [len(epochs.ch_names)*[0]]
#                         for i, tr in enumerate(epochs)]
# =============================================================================

        # Plot visual rejection
        fig, ax = plt.subplots(2, 2, figsize=[14, 7], sharex='col',
                               sharey='row', frameon=False)
        fig.suptitle(info + ' Fp: %i' % fp, fontsize=16)

        # fig 1: Heatmap of x = trials, y = channels, z = max amplitude
        z1 = np.transpose(np.array(rng_heat))  # channels times trials
        x1 = trIndx
        y1 = [epochs.ch_names[ch] for ch in chIndx]  # names as y axis
        ax[0, 0].imshow(z1, aspect=len(x1) / len(y1) / 2)
        ax[0, 0].set_yticks(np.arange(len(y1)))
        ax[0, 0].set_yticklabels(y1)
        ax[0, 0].tick_params(labelbottom=True)
        ax[0, 0].invert_yaxis()

        # fig 2: Max ptp amplitude per channel across trials
        x2 = np.array([max([trial[ch] * 1e6 for trial in rng_ch])
                       for ch in range(len(chIndx))])
        y2 = chIndx
        ax[0, 1].scatter(x2, y2, color='blue', s=0.8)
        ax[0, 1].tick_params(labelbottom=True, labelleft=True)
        reject_channels = Highlighter(ax[0, 1], x2, y2)

        # fig 3: Max amplitude per trial across channels
        x3 = trIndx
        y3 = np.array([max(trial) * 1e6 for trial in rng_ch])
        ax[1, 0].scatter(x3, y3, color='blue', s=0.8)
        reject_trials = Highlighter(ax[1, 0], x3, y3)

        plt.show()

        # Remove selected trials from trial index
        selected_trials = x3[reject_trials.mask]
        selected_channels = y2[reject_channels.mask]
        trIndx = np.setdiff1d(trIndx, selected_trials)
        chIndx = np.setdiff1d(chIndx, selected_channels)
        bad_trials += list(selected_trials)
        bad_trials.sort()
        # bad_trials, trIndx = toggle_x(selected_trials, trIndx,
        #                               bad_trials)
        # bad_channels, chIndx = toggle_x(selected_channels, chIndx,
        #                                bad_channels)

        print('All selected trials:', bad_trials)
        # print('All selected channels:', bad_channels)
        print('%i trials selected: %s' %
              (len(selected_trials), ' '.join(str(trial) for trial in
                                              selected_trials)))
        print('%i channels selected: %s' %
              (len(selected_channels), ' '.join(str(trial) for trial in
                                                selected_channels)))
        print('Trials left:', len(trIndx))
        print('Channels left:', len(chIndx))
    return bad_trials
